/* DirectorySync application in Genie/Gtk */
[indent=2]
uses
  Gtk
  Vte
  Posix

class SyncWindow : Window
  term : Terminal
  mainbox : HBox
  confbox : HBox
  configdir : string
  local : string
  remote : string
  _cancel : Button
  _local : Entry
  _remote : Entry
  _unlink : CheckButton
  _pid : int

  def load()
    try
      var unlink = ""
      FileUtils.get_contents (configdir+"local", out local)
      FileUtils.get_contents (configdir+"remote", out remote)
      FileUtils.get_contents (configdir+"unlink", out unlink)
      _local.text = local
      _remote.text = remote
      _unlink.active = unlink.to_bool ()
      if !FileUtils.test (local, FileTest.IS_DIR)
        raise new FileError.EXIST ("Does not exist")
      do_main ()
    except e: FileError
      do_config ()

  def save()
    try
      FileUtils.set_contents (configdir+"local", _local.text)
      FileUtils.set_contents (configdir+"remote", _remote.text)
      FileUtils.set_contents (configdir+"unlink", _unlink.active.to_string ())
    except e: FileError
      print "Fail to save\n"
      do_config ()
    load ()

  def cmd_start(cmd: array of string)
    if _pid != -1
      cmd_cancel()
    _cancel.show ()
    _pid = term.fork_command (cmd[0], cmd, null, local, false, false, false)
    term.has_focus = true

  def cmd_cancel()
    if _pid != -1
      Posix.kill (_pid, Posix.SIGINT)
      Posix.kill (_pid, Posix.SIGKILL)
      _pid = -1
    cmd_end ()

  def cmd_end()
    _cancel.hide ()
    _pid = -1

  def do_files()
    print ("list files, select, open, remove")
    cmd_start({"ls"})

  def do_push()
    if _unlink.active
      cmd_start ({"rsync", "--delete", "-avz", local+"/", remote})
    else
      cmd_start ({"rsync", "-avz", local+"/", remote})

  def do_pull()
    if _unlink.active
      cmd_start ({"rsync", "--delete", "-avz", remote+"/", local})
    else
      cmd_start ({"rsync", "-avz", remote+"/", local})

  def do_shell()
    cmd_start ({"sh"})

  def do_config()
    show_all ()
    confbox.show_all () 
    mainbox.hide_all () 

  def do_main()
    show_all ()
    confbox.hide_all () 
    mainbox.show_all () 
    if _pid != -1
      _cancel.show ()
    else
      _cancel.hide ()

  init
    _pid = -1
    title = "DSync"
    destroy.connect (Gtk.main_quit)
    configdir = Environment.get_home_dir () + "/.config/dsync/"
    DirUtils.create_with_parents (configdir, 0755)

    /* configuration window */
    confbox = new HBox (false, 0)
    var button_back = new Button.with_label ("  Ok  ")
    button_back.clicked.connect (save)
    var leftbox = new VBox (false, 2)
    leftbox.pack_start (new Label ("Configuration"), false, false, 5)
    leftbox.pack_start (new HSeparator (), false, false, 2)
    _local = new Entry ()
    _local.text = local
    _local.activate.connect (save)
    leftbox.pack_start (new Label ("Local directory:"), false, false, 2)
    leftbox.pack_start (_local, false, false, 2)
    _remote = new Entry ()
    _remote.text = remote
    _remote.activate.connect (save)
    leftbox.pack_start (new Label ("Remote directory:"), false, false, 2)
    leftbox.pack_start (_remote, false, false, 2)
    _unlink = new CheckButton.with_label ("Delete untracked files")
    leftbox.pack_start (_unlink, false, false, 3)
    confbox.add (leftbox)
    confbox.pack_end (button_back, false, false, 3)

    /* main window */
    mainbox = new HBox (false, 0)
    var vbox = new VBox (false, 2)
    term = new Terminal ()
    term.resize_window (800,600)
    term.child_exited.connect (cmd_end)
    var termbox = new VBox (false, 0)
    termbox.add (term)
    _cancel = new Button.with_label ("Cancel");
    _cancel.clicked.connect (cmd_cancel)
    termbox.pack_end (_cancel, false, false, 0);
    mainbox.add (termbox)

    var button_push = new Button.with_label ("Push");
    button_push.clicked.connect (do_push)
    vbox.add (button_push)

    var button_pull = new Button.with_label ("Pull");
    button_pull.clicked.connect (do_pull)
    vbox.add (button_pull)

    var button_shell = new Button.with_label ("Shell");
    button_shell.clicked.connect (do_shell)
    vbox.add (button_shell)

    var button_config = new Button.with_label ("Config");
    button_config.clicked.connect (do_config)
    vbox.add (button_config)

    mainbox.pack_end (vbox, false, false, 1)

    var bigbox = new VBox (false, 0)
    bigbox.add (confbox)
    bigbox.add (mainbox)
    add (bigbox)

init
  Gtk.init (ref args)
  new SyncWindow ().load ()
  Gtk.main ()
